#encode: utf-8
'''
Created on 07/11/2013
@copyright:  2013 Daniel Santana
@todo: Better handling for dwg import units
@author: Daniel Santana
@requires: PySide
@note: Install file running the this file, using python.executeFile "path to file"
@bug: Previous version didn't have the FPValue.Get for keys.
'''
from PySide import QtGui
import MaxPlus
from os import path
import ConfigParser

MaxPlus.INode.ColorByLayer = MaxPlus._swig_property(lambda self: MaxPlus.Core.EvalMAXScript("$'%s'.ColorByLayer" % self.Name),
                                                    lambda self, x: MaxPlus.Core.EvalMAXScript("$'%s'.ColorByLayer = %s;true" % (self.Name, x)))


class DwgImporter(object):
    '''
    Imports all the selected files and creates layers for each one.
    '''

    def delete_empty_layers(self):
        '''
        Deletes all empty layers.
        '''
        delete_me = []
        for i in xrange(MaxPlus.LayerManager.GetNumLayers()):
            layer = MaxPlus.LayerManager.GetLayer(i)
            if len(layer.GetNodes()) == 0 and layer.GetName() != '0':
                delete_me.append(layer.GetName())
        for i in delete_me:
            MaxPlus.LayerManager.DeleteLayer(MaxPlus.WStr(i))

    def create_macro(self):
        mxs_macro = '''
macroScript dsCleanDWG
    category:"dsScripts"
    buttonText:"Import and Clean CAD"
    toolTip:"Import's and cleans dwgs"
(
    on execute do (
        if python != undefined do (
            python.execute "
import sys
sys.dont_write_bytecode = True
try:
    import PySide
except ImportError:
    import site
    site.addsitedir('C:\Python27\Lib\site-packages')
"
python.executefile @"''' + __file__ + '''"
        )
    )
)
'''
        MaxPlus.Core.EvalMAXScript(mxs_macro)
        del mxs_macro

    def create_layer(self, name, nodes=None):
        '''
        Creates a new layer if required and add's nodes if passed.
        @var name: Layer name
        @type name: str
        @var nodes: Nodes to be added
        @type nodes: list(MaxPlus.INode)
        @return: Layer
        @rtype: MaxPlus.Layer
        '''
        layer = None
        try:
            layer = MaxPlus.LayerManager.GetLayer(name)
        except:
            layer = MaxPlus.LayerManager.CreateLayer(name)
        if nodes:
            for node in nodes:
                layer.AddToLayer(node)
        layer.SetShowFrozenWithMtl(False)
        return layer

    def run(self, no_options=False):
        '''
        Loads all the files and imports into layers
        '''
        try:
            # Tries to set unique names for dwg's imports
            plug_cfg = MaxPlus.PathManager.GetPlugcfgDir()
            dwg = ConfigParser.SafeConfigParser(path.join(plug_cfg, 'dwg_dxf_import.ini'))
            dwg.set('LastUsedSettings', 'GiveNewNodesUniqueNames', 'ON')
            with file(path.join(plug_cfg, 'dwg_dxf_import.ini'), 'w') as f:
                dwg.write(f)
        except:
            pass

        files = QtGui.QFileDialog.getOpenFileNames(
                                        caption="Select files to import",
                                        dir=MaxPlus.PathManager.GetImportDir(),
                                        filter='DWG Files (*.dwg)')
        # Let's get the current scene nodes so we don't screw up existing objects
        scene_nodes = [node for node in MaxPlus.Core.GetRootNode().Children if not node.IsFrozen() and not node.IsHidden()]
        for f in files[0]:
            print 'Importing %s' % path.basename(f)
            if MaxPlus.FileManager.Import(f, no_options):
                nodes = [node for node in MaxPlus.Core.GetRootNode().Children
                         if not node.IsFrozen() and not node.IsHidden() and node not in scene_nodes]
                for node in nodes:
                    node.ShowFrozenWithMtl = False
                    node.ColorByLayer = False
                layer = self.create_layer(path.splitext(path.basename(f))[0],
                                          nodes)
                layer.Freeze(True)
        self.delete_empty_layers()

if __name__ == '__main__':
    app = QtGui.QApplication.instance()
    if not app:
        app = QtGui.QApplication([])
    importer = DwgImporter()
    if MaxPlus.Core.EvalMAXScript('keyboard.controlPressed').Get():
        importer.create_macro()
    no_options = MaxPlus.Core.EvalMAXScript('keyboard.shiftPressed').Get()
    importer.run(no_options)
